/* generated with mime-types-include.sh, don't edit */
G_GNUC_UNUSED static const gchar *mime_types[] = {
"application/mxf",
"application/ogg",
"application/ram",
"application/sdp",
"application/smil",
"application/smil+xml",
"application/vnd.apple.mpegurl",
"application/vnd.ms-asf",
"application/vnd.ms-wpl",
"application/vnd.rn-realmedia",
"application/vnd.rn-realmedia-vbr",
"application/x-extension-m4a",
"application/x-extension-mp4",
"application/x-flac",
"application/x-flash-video",
"application/x-matroska",
"application/x-netshow-channel",
"application/x-ogg",
"application/x-quicktime-media-link",
"application/x-quicktimeplayer",
"application/x-shorten",
"application/x-smil",
"application/xspf+xml",
"audio/3gpp",
"audio/3gpp2",
"audio/aac",
"audio/ac3",
"audio/AMR",
"audio/AMR-WB",
"audio/basic",
"audio/dv",
"audio/eac3",
"audio/flac",
"audio/m4a",
"audio/midi",
"audio/mp1",
"audio/mp2",
"audio/mp3",
"audio/mp4",
"audio/mpeg",
"audio/mpegurl",
"audio/mpg",
"audio/ogg",
"audio/opus",
"audio/prs.sid",
"audio/scpls",
"audio/vnd.rn-realaudio",
"audio/wav",
"audio/webm",
"audio/x-aac",
"audio/x-aiff",
"audio/x-ape",
"audio/x-flac",
"audio/x-gsm",
"audio/x-it",
"audio/x-m4a",
"audio/x-m4b",
"audio/x-matroska",
"audio/x-mod",
"audio/x-mp1",
"audio/x-mp2",
"audio/x-mp3",
"audio/x-mpg",
"audio/x-mpeg",
"audio/x-mpegurl",
"audio/x-ms-asf",
"audio/x-ms-asx",
"audio/x-ms-wax",
"audio/x-ms-wma",
"audio/x-musepack",
"audio/x-opus+ogg",
"audio/x-pn-aiff",
"audio/x-pn-au",
"audio/x-pn-realaudio",
"audio/x-pn-realaudio-plugin",
"audio/x-pn-wav",
"audio/x-pn-windows-acm",
"audio/x-realaudio",
"audio/x-real-audio",
"audio/x-s3m",
"audio/x-sbc",
"audio/x-scpls",
"audio/x-shorten",
"audio/x-speex",
"audio/x-stm",
"audio/x-tta",
"audio/x-wav",
"audio/x-wavpack",
"audio/x-vorbis",
"audio/x-vorbis+ogg",
"audio/x-xm",
"image/vnd.rn-realpix",
"image/x-pict",
"misc/ultravox",
"text/google-video-pointer",
"text/x-google-video-pointer",
"video/3gp",
"video/3gpp",
"video/3gpp2",
"video/dv",
"video/divx",
"video/fli",
"video/flv",
"video/mp2t",
"video/mp4",
"video/mp4v-es",
"video/mpeg",
"video/mpeg-system",
"video/msvideo",
"video/ogg",
"video/quicktime",
"video/vivo",
"video/vnd.divx",
"video/vnd.mpegurl",
"video/vnd.rn-realvideo",
"video/vnd.vivo",
"video/webm",
"video/x-anim",
"video/x-avi",
"video/x-flc",
"video/x-fli",
"video/x-flic",
"video/x-flv",
"video/x-m4v",
"video/x-matroska",
"video/x-mjpeg",
"video/x-mpeg",
"video/x-mpeg2",
"video/x-ms-asf",
"video/x-ms-asf-plugin",
"video/x-ms-asx",
"video/x-msvideo",
"video/x-ms-wm",
"video/x-ms-wmv",
"video/x-ms-wmx",
"video/x-ms-wvx",
"video/x-nsv",
"video/x-ogm+ogg",
"video/x-theora",
"video/x-theora+ogg",
"video/x-totem-stream",
NULL
};
G_GNUC_UNUSED static const gchar *audio_mime_types[] = {
"audio/3gpp",
"audio/3gpp2",
"audio/aac",
"audio/ac3",
"audio/AMR",
"audio/AMR-WB",
"audio/basic",
"audio/dv",
"audio/eac3",
"audio/flac",
"audio/m4a",
"audio/midi",
"audio/mp1",
"audio/mp2",
"audio/mp3",
"audio/mp4",
"audio/mpeg",
"audio/mpg",
"audio/ogg",
"audio/opus",
"audio/prs.sid",
"audio/scpls",
"audio/vnd.rn-realaudio",
"audio/wav",
"audio/webm",
"audio/x-aac",
"audio/x-aiff",
"audio/x-ape",
"audio/x-flac",
"audio/x-gsm",
"audio/x-it",
"audio/x-m4a",
"audio/x-m4b",
"audio/x-matroska",
"audio/x-mod",
"audio/x-mp1",
"audio/x-mp2",
"audio/x-mp3",
"audio/x-mpg",
"audio/x-mpeg",
"audio/x-ms-asf",
"audio/x-ms-asx",
"audio/x-ms-wax",
"audio/x-ms-wma",
"audio/x-musepack",
"audio/x-opus+ogg",
"audio/x-pn-aiff",
"audio/x-pn-au",
"audio/x-pn-wav",
"audio/x-pn-windows-acm",
"audio/x-realaudio",
"audio/x-real-audio",
"audio/x-s3m",
"audio/x-sbc",
"audio/x-shorten",
"audio/x-speex",
"audio/x-stm",
"audio/x-tta",
"audio/x-wav",
"audio/x-wavpack",
"audio/x-vorbis",
"audio/x-vorbis+ogg",
"audio/x-xm",
"application/x-flac",
NULL
};
G_GNUC_UNUSED static const gchar *video_mime_types[] = {
"application/mxf",
"application/ogg",
"application/ram",
"application/sdp",
"application/vnd.apple.mpegurl",
"application/vnd.ms-asf",
"application/vnd.ms-wpl",
"application/vnd.rn-realmedia",
"application/vnd.rn-realmedia-vbr",
"application/x-extension-m4a",
"application/x-extension-mp4",
"application/x-flash-video",
"application/x-matroska",
"application/x-netshow-channel",
"application/x-ogg",
"application/x-quicktimeplayer",
"application/x-shorten",
"image/vnd.rn-realpix",
"image/x-pict",
"misc/ultravox",
"text/x-google-video-pointer",
"video/3gp",
"video/3gpp",
"video/3gpp2",
"video/dv",
"video/divx",
"video/fli",
"video/flv",
"video/mp2t",
"video/mp4",
"video/mp4v-es",
"video/mpeg",
"video/mpeg-system",
"video/msvideo",
"video/ogg",
"video/quicktime",
"video/vivo",
"video/vnd.divx",
"video/vnd.mpegurl",
"video/vnd.rn-realvideo",
"video/vnd.vivo",
"video/webm",
"video/x-anim",
"video/x-avi",
"video/x-flc",
"video/x-fli",
"video/x-flic",
"video/x-flv",
"video/x-m4v",
"video/x-matroska",
"video/x-mjpeg",
"video/x-mpeg",
"video/x-mpeg2",
"video/x-ms-asf",
"video/x-ms-asf-plugin",
"video/x-ms-asx",
"video/x-msvideo",
"video/x-ms-wm",
"video/x-ms-wmv",
"video/x-ms-wmx",
"video/x-ms-wvx",
"video/x-nsv",
"video/x-ogm+ogg",
"video/x-theora",
"video/x-theora+ogg",
"video/x-totem-stream",
"audio/x-pn-realaudio",
NULL
};
